"""Boehr-Mur algoritm. Adapted from the

Cormen, Thomas H. and Charles E. Leiserson and Ronald L. Rivest
Introduction to Algorithms, 34.5
"""

def _lastOccurence(pattern):
	last = 256 * [-1]
	for i in xrange(0, len(pattern)):
		last[ord(pattern[i])] = i
	return last

def _goodSuffix(string):
	prefix = _prefixFunction(string)
	invprefix = _prefixFunction(_inverse(string))

	goodsuffix = len(string) * [len(string) - prefix[len(string) - 1]]

	for i in xrange(0, len(string)):
		j = len(string) - invprefix[i] - 1
		if goodsuffix[j] > i - invprefix[i]:
			goodsuffix[j] = i - invprefix[i]

	return goodsuffix

def _prefixFunction(string):
	prefix = len(string) * [0]
	k = 0
	for q in xrange(1, len(string)):
		while k > 0 and string[k] != string[q]:
			k = prefix[k - 1]
		if string[k] == string[q]:
			k += 1
		prefix[q] = k
	return prefix

def _inverse(string):
	res = ""
	for s in string:
		res = s + res
	return res

class Matcher(object):

	def __init__(self, string, pattern):
		self.__string = string
		self.__pattern = pattern
		self.__checkedPrefixLength = 0

		self.__last = _lastOccurence(self.__pattern)
		self.__goodSuffix = _goodSuffix(self.__pattern)

	def match(self, prefixLength):
		try:
			s = self.__checkedPrefixLength
			while s < prefixLength:
				j = len(self.__pattern) - 1
				while True:
					if j < 0:
						self.__checkedPrefixLength = s + 1
						return s
					cs = self.__string[s + j]
					if cs != self.__pattern[j]:
						break
					j -= 1
				s += max(self.__goodSuffix[j], j - self.__last[ord(cs)])
			self.__checkedPrefixLength = s
			return None
		except IndexError:
			self.__checkedPrefixLength = prefixLength
			return None

	def skip(self, prefixLength):
		self.__checkedPrefixLength = prefixLength
