import logging, os.path, os


class ColoredFormatter(logging.Formatter):
    def __init__(self, fmt, datefmt = None):
        logging.Formatter.__init__(self, fmt, datefmt)

    def format(self, record):
        return "%s%s%s\x1b[0;39m" \
            % (self.levelColor(record.levelno), \
               logging.Formatter.format(self, record), \
               self.levelBold(record.levelno))

    def levelBold(self, level):
        if level >= logging.FATAL: return "\x1b[1m"
        return ""

    def levelColor(self, level):
        if level >= logging.ERROR: return "\x1b[31m"
        if level >= logging.WARN: return "\x1b[33m"
        if level >= logging.INFO: return "\x1b[37m"
        return "\x1b[32m"


class FileHandler(logging.FileHandler):
    def __init__(self, filename, mode="a", permissions = 0600):
        old_umask = os.umask(0777)
        logging.FileHandler.__init__(self, filename, mode)
        os.umask(old_umask)
        os.chmod(filename, permissions)


def initLog(component, dir, debug_mode = False):
    handler = FileHandler(os.path.join(dir, component + ".log"))
    handler.setFormatter(logging.Formatter("%(asctime)s %(levelname)s %(message)s"))

    root = logging.getLogger()

    root.addHandler(handler)
    root.setLevel(debug_mode and logging.DEBUG or logging.INFO)
